@@ -12,10 +12,11 @@ pub mod database;
 pub mod encryption;
 pub mod history;
 pub mod import;
-pub mod kv;
 pub mod login;
 pub mod logout;
+pub mod meta;
 pub mod ordering;
+pub mod plugin;
 pub mod record;
 pub mod register;
 pub mod secrets;
