@@ -1,6 +1,5 @@
 use clap::Parser;
-use eyre::{bail, Result};
-use tokio::{fs::File, io::AsyncWriteExt};
+use eyre::{Result, bail};
 
 use atuin_client::{api_client, settings::Settings};
 
@@ -18,15 +17,15 @@ pub struct Cmd {
 
 impl Cmd {
     pub async fn run(self, settings: &Settings) -> Result<()> {
-        run(settings, &self.username, &self.email, &self.password).await
+        run(settings, self.username, self.email, self.password).await
     }
 }
 
 pub async fn run(
     settings: &Settings,
-    username: &Option<String>,
-    email: &Option<String>,
-    password: &Option<String>,
+    username: Option<String>,
+    email: Option<String>,
+    password: Option<String>,
 ) -> Result<()> {
     use super::login::or_user_input;
     println!("Registering for an Atuin Sync account");
@@ -45,11 +44,17 @@ pub async fn run(
     let session =
         api_client::register(settings.sync_address.as_str(), &username, &email, &password).await?;
 
-    let path = settings.session_path.as_str();
-    let mut file = File::create(path).await?;
-    file.write_all(session.session.as_bytes()).await?;
+    let meta = Settings::meta_store().await?;
+    meta.save_session(&session.session).await?;
 
     let _key = atuin_client::encryption::load_key(settings)?;
 
+    println!(
+        "Registration successful! Please make a note of your key (run 'atuin key') and keep it safe."
+    );
+    println!(
+        "You will need it to log in on other devices, and we cannot help recover it if you lose it."
+    );
+
     Ok(())
 }
