@@ -1,26 +1,22 @@
-use std::path::PathBuf;
-
 use crate::{SHA, VERSION};
 use atuin_client::{api_client, database::Database, settings::Settings};
 use colored::Colorize;
 use eyre::{Result, bail};
 
 pub async fn run(settings: &Settings, db: &impl Database) -> Result<()> {
-    let session_path = settings.session_path.as_str();
-
-    if !PathBuf::from(session_path).exists() {
+    if !settings.logged_in().await? {
         bail!("You are not logged in to a sync server - cannot show sync status");
     }
 
     let client = api_client::Client::new(
         &settings.sync_address,
-        settings.session_token()?.as_str(),
+        settings.session_token().await?.as_str(),
         settings.network_connect_timeout,
         settings.network_timeout,
     )?;
 
-    let status = client.status().await?;
-    let last_sync = Settings::last_sync()?;
+    let me = client.me().await?;
+    let last_sync = Settings::last_sync().await?;
 
     println!("Atuin v{VERSION} - Build rev {SHA}\n");
 
@@ -42,7 +38,7 @@ pub async fn run(settings: &Settings, db: &impl Database) -> Result<()> {
     if settings.auto_sync {
         println!("{}", "[Remote]".green());
         println!("Address: {}", settings.sync_address);
-        println!("Username: {}", status.username);
+        println!("Username: {}", me.username);
     }
 
     Ok(())
